% Copyright (C) 2025 by Raehyun Kim (rhkim79@math.berkeley.edu)
function [root, info] = efficient_bisect(func, Int, params)
% initial setting
a = Int.a; b = Int.b;
r_tol = params.root_tol;
f_tol = params.func_tol;

fa = func(a); fb = func(b);

% initialize output
root = NaN;
info.flag = 1; 

% condition check
if (fa*fb > 0)
    disp('Wrong Input')
    return
end

while (abs(a-b) > r_tol)
    root = (a+b)/2; fr = func(root);
    if abs(fr) < f_tol
        info.flag = 0;
        return
    end

    if fr * fa < 0
        b = root;
        % instead of fb = func(root)
        fb = fr;
    else
        a = root;
        fa = fr;
    end

    %%
    % x0 = x1; f0 = f1;
    % x1 = x2; f1 = f2;
    % x2 = x3; f2 = f3;
end

info.flag = 0;
return
end