% Copyright (C) 2025 by Raehyun Kim (rhkim79@math.berkeley.edu)
efficient_flag = 1;
if efficient_flag
    disp('EFFICIENT BISECTION')
else
    disp('INEFFICIENT BISECTION')
end
func1 = @(x) x.*exp(-x) - 2*x + 1;            Int1.a = 0; Int1.b = 3;
func2 = @(x) x.*cos(x) - 2*x.^2 + 3*x - 1;    Int2.a = 1; Int2.b = 3;
func3 = @(x) x.^3-7*x.^2+14*x-6;              Int3.a = 0; Int3.b = 1;
func4 = @(x) sqrt(x)-cos(x);                  Int4.a = 0; Int4.b = 1;
func5 = @(x) 2*x.*cos(2*x) - (x+1).^2;        Int5.a =-4; Int5.b =-2;


test_functions = {func1, func2, func3, func4, func5};
test_intervals = {Int1, Int2, Int3, Int4, Int5};

params.root_tol = 1e-7; params.func_tol = 1e-7;


for j = 1:5
    func = test_functions{j}; Int = test_intervals{j};
    disp("#####")
    profile on
    %%%%
    % PUT YOUR FUNCTION NAME BELOW e.g.
    % [root, info] = modifiedzeroin31415926535(func, Int, params)
    %%%%
    if efficient_flag
        [root, info] = efficient_bisect(func, Int, params); 
    else
        [root, info] = inefficient_bisect(func, Int, params); 
    end
    profile off
    % Verify root
    root_tol_check = abs(root - fzero(func, root)) < params.root_tol;
    func_tol_check = abs(func(root)) < params.func_tol;
    int_check = (root > Int.a) & (root < Int.b);
    if (root_tol_check || func_tol_check) && int_check
        fprintf("Find root CORRECTLY for test function %d !! \n", j);
    else
        fprintf("Failed for test function%d !! \n", j);
    end
    % check # of function calls
    p = profile('info');
    foo = {p.FunctionTable.CompleteName};
    bar = strfind(foo, func2str(func));
    fcall_idx = find(~cellfun(@isempty, bar));
    num_fcall = p.FunctionTable(fcall_idx).NumCalls;
    fprintf("It takes %d function calls !! \n", num_fcall);
end
